#include "Turnstile.h"
#include "Turnstile_sm.h"


// Static class declarations.
TurnstileFSM_Idle TurnstileFSM::Idle("TurnstileFSM::Idle", 0);
TurnstileFSM_Active TurnstileFSM::Active("TurnstileFSM::Active", 1);
TurnstileFSM_WaitingAppA TurnstileFSM::WaitingAppA("TurnstileFSM::WaitingAppA", 2);
TurnstileFSM_WaitingAppB TurnstileFSM::WaitingAppB("TurnstileFSM::WaitingAppB", 3);
TurnstileFSM_ClosingMessage TurnstileFSM::ClosingMessage("TurnstileFSM::ClosingMessage", 4);
TurnstileFSM_TurnstileForced TurnstileFSM::TurnstileForced("TurnstileFSM::TurnstileForced", 5);

void TurnstileState::evTimeout(TurnstileContext& context)
{
    Default(context);
    return;
}

void TurnstileState::evTokenIoError(TurnstileContext& context)
{
    Default(context);
    return;
}

void TurnstileState::evTokenReadEnd(TurnstileContext& context, string& data)
{
    Default(context);
    return;
}

void TurnstileState::evTokenSelected(TurnstileContext& context)
{
    Default(context);
    return;
}

void TurnstileState::evTokenWriteEnd(TurnstileContext& context)
{
    Default(context);
    return;
}

void TurnstileState::evTurnstileForced(TurnstileContext& context)
{
    Default(context);
    return;
}

void TurnstileState::evTurnstileIdle(TurnstileContext& context)
{
    Default(context);
    return;
}

void TurnstileState::Default(TurnstileContext& context)
{
    throw (
        TransitionUndefinedException(
            context.getState().getName(),
            context.getTransition()));

    return;
}

void TurnstileFSM_Idle::Entry(TurnstileContext& context)

{
    Turnstile& ctxt = context.getOwner();

    ctxt.lcd.showMsg(MSG_IDLE, NULL);
    return;
}

void TurnstileFSM_Idle::evTokenSelected(TurnstileContext& context)
{

    (context.getState()).Exit(context);
    context.setState(TurnstileFSM::WaitingAppA);
    (context.getState()).Entry(context);

    return;
}

void TurnstileFSM_Idle::evTurnstileForced(TurnstileContext& context)
{

    (context.getState()).Exit(context);
    context.setState(TurnstileFSM::TurnstileForced);
    (context.getState()).Entry(context);

    return;
}

void TurnstileFSM_Active::evTokenIoError(TurnstileContext& context)
{
    Turnstile& ctxt = context.getOwner();

    (context.getState()).Exit(context);
    context.clearState();
    try
    {
        ctxt.lcd.showMsg(MSG_TOKEN_ERROR, NULL);
        context.setState(TurnstileFSM::ClosingMessage);
    }
    catch (...)
    {
        context.setState(TurnstileFSM::ClosingMessage);
        throw;
    }
    (context.getState()).Entry(context);

    return;
}

void TurnstileFSM_Active::evTurnstileForced(TurnstileContext& context)
{

    (context.getState()).Exit(context);
    context.setState(TurnstileFSM::TurnstileForced);
    (context.getState()).Entry(context);

    return;
}

void TurnstileFSM_WaitingAppA::evTokenReadEnd(TurnstileContext& context, string& data)
{
    Turnstile& ctxt = context.getOwner();

    (context.getState()).Exit(context);
    context.clearState();
    try
    {
        ctxt.card.readApp(APP_SVSS4_A, data);
        context.setState(TurnstileFSM::WaitingAppB);
    }
    catch (...)
    {
        context.setState(TurnstileFSM::WaitingAppB);
        throw;
    }
    (context.getState()).Entry(context);

    return;
}

void TurnstileFSM_WaitingAppB::evTokenWriteEnd(TurnstileContext& context)
{

    (context.getState()).Exit(context);
    context.setState(TurnstileFSM::Idle);
    (context.getState()).Entry(context);

    return;
}

void TurnstileFSM_ClosingMessage::Entry(TurnstileContext& context)

{
    Turnstile& ctxt = context.getOwner();

    ctxt.timer.start(2000);
    return;
}

void TurnstileFSM_ClosingMessage::evTimeout(TurnstileContext& context)
{

    (context.getState()).Exit(context);
    context.setState(TurnstileFSM::Idle);
    (context.getState()).Entry(context);

    return;
}

void TurnstileFSM_TurnstileForced::Entry(TurnstileContext& context)

{
    Turnstile& ctxt = context.getOwner();

    ctxt.lcd.showMsg(MSG_TURNSTILE_FORCED, NULL);
    return;
}

void TurnstileFSM_TurnstileForced::evTurnstileForced(TurnstileContext& context)
{


    return;
}

void TurnstileFSM_TurnstileForced::evTurnstileIdle(TurnstileContext& context)
{

    (context.getState()).Exit(context);
    context.setState(TurnstileFSM::Idle);
    (context.getState()).Entry(context);

    return;
}
