#ifndef _H_TURNSTILE_SM
#define _H_TURNSTILE_SM

#define SMC_USES_IOSTREAMS

#include "statemap.h"

// Forward declarations.
class TurnstileFSM;
class TurnstileFSM_Idle;
class TurnstileFSM_Active;
class TurnstileFSM_WaitingAppA;
class TurnstileFSM_WaitingAppB;
class TurnstileFSM_ClosingMessage;
class TurnstileFSM_TurnstileForced;
class TurnstileFSM_Default;
class TurnstileState;
class TurnstileContext;
class Turnstile;

class TurnstileState :
    public State
{
public:

    TurnstileState(const char *name, int stateId)
    : State(name, stateId)
    {};

    virtual void Entry(TurnstileContext&) {};
    virtual void Exit(TurnstileContext&) {};

    virtual void evTimeout(TurnstileContext& context);
    virtual void evTokenIoError(TurnstileContext& context);
    virtual void evTokenReadEnd(TurnstileContext& context, string& data);
    virtual void evTokenSelected(TurnstileContext& context);
    virtual void evTokenWriteEnd(TurnstileContext& context);
    virtual void evTurnstileForced(TurnstileContext& context);
    virtual void evTurnstileIdle(TurnstileContext& context);

protected:

    virtual void Default(TurnstileContext& context);
};

class TurnstileFSM
{
public:

    static TurnstileFSM_Idle Idle;
    static TurnstileFSM_Active Active;
    static TurnstileFSM_WaitingAppA WaitingAppA;
    static TurnstileFSM_WaitingAppB WaitingAppB;
    static TurnstileFSM_ClosingMessage ClosingMessage;
    static TurnstileFSM_TurnstileForced TurnstileForced;
};

class TurnstileFSM_Default :
    public TurnstileState
{
public:

    TurnstileFSM_Default(const char *name, int stateId)
    : TurnstileState(name, stateId)
    {};

};

class TurnstileFSM_Idle :
    public TurnstileFSM_Default
{
public:
    TurnstileFSM_Idle(const char *name, int stateId)
    : TurnstileFSM_Default(name, stateId)
    {};

    void Entry(TurnstileContext&);
    void evTokenSelected(TurnstileContext& context);
    void evTurnstileForced(TurnstileContext& context);
};

class TurnstileFSM_Active :
    public TurnstileFSM_Default
{
public:
    TurnstileFSM_Active(const char *name, int stateId)
    : TurnstileFSM_Default(name, stateId)
    {};

    void evTokenIoError(TurnstileContext& context);
    void evTurnstileForced(TurnstileContext& context);
};

class TurnstileFSM_WaitingAppA :
    public TurnstileFSM_Default
{
public:
    TurnstileFSM_WaitingAppA(const char *name, int stateId)
    : TurnstileFSM_Default(name, stateId)
    {};

    void evTokenReadEnd(TurnstileContext& context, string& data);
};

class TurnstileFSM_WaitingAppB :
    public TurnstileFSM_Default
{
public:
    TurnstileFSM_WaitingAppB(const char *name, int stateId)
    : TurnstileFSM_Default(name, stateId)
    {};

    void evTokenWriteEnd(TurnstileContext& context);
};

class TurnstileFSM_ClosingMessage :
    public TurnstileFSM_Default
{
public:
    TurnstileFSM_ClosingMessage(const char *name, int stateId)
    : TurnstileFSM_Default(name, stateId)
    {};

    void Entry(TurnstileContext&);
    void evTimeout(TurnstileContext& context);
};

class TurnstileFSM_TurnstileForced :
    public TurnstileFSM_Default
{
public:
    TurnstileFSM_TurnstileForced(const char *name, int stateId)
    : TurnstileFSM_Default(name, stateId)
    {};

    void Entry(TurnstileContext&);
    void evTurnstileForced(TurnstileContext& context);
    void evTurnstileIdle(TurnstileContext& context);
};

class TurnstileContext :
    public FSMContext
{
public:

    TurnstileContext(Turnstile& owner)
    : _owner(owner)
    {
        setState(TurnstileFSM::Idle);
        TurnstileFSM::Idle.Entry(*this);
    };

    Turnstile& getOwner() const
    {
        return (_owner);
    };

    TurnstileState& getState() const
    {
        if (_state == NULL)
        {
            throw StateUndefinedException();
        }

        return (dynamic_cast<TurnstileState&>(*_state));
    };

    void evTimeout()
    {
        (getState()).evTimeout(*this);
    };

    void evTokenIoError()
    {
        (getState()).evTokenIoError(*this);
    };

    void evTokenReadEnd(string& data)
    {
        (getState()).evTokenReadEnd(*this, data);
    };

    void evTokenSelected()
    {
        (getState()).evTokenSelected(*this);
    };

    void evTokenWriteEnd()
    {
        (getState()).evTokenWriteEnd(*this);
    };

    void evTurnstileForced()
    {
        (getState()).evTurnstileForced(*this);
    };

    void evTurnstileIdle()
    {
        (getState()).evTurnstileIdle(*this);
    };

private:

    Turnstile& _owner;
};

#endif
